/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.commons.dwr.convert;

import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import org.directwebremoting.convert.BaseV20Converter;
import org.directwebremoting.dwrp.ParseUtil;
import org.directwebremoting.dwrp.ProtocolConstants;
import org.directwebremoting.extend.ConverterManager;
import org.directwebremoting.extend.InboundContext;
import org.directwebremoting.extend.InboundVariable;
import org.directwebremoting.extend.MarshallException;
import org.directwebremoting.extend.OutboundContext;
import org.directwebremoting.extend.OutboundVariable;
import org.directwebremoting.util.LocalUtil;
import org.directwebremoting.util.Messages;

public abstract class AbstractConverter extends BaseV20Converter {

  protected Map<String, String> extractInboundTokens(Class<?> paramType, String value)
      throws MarshallException {
    Map<String, String> tokens = new HashMap<String, String>();
    StringTokenizer st = new StringTokenizer(value, ProtocolConstants.INBOUND_MAP_SEPARATOR);
    int size = st.countTokens();

    for (int i = 0; i < size; i++) {
      String token = st.nextToken();
      if (token.trim().length() == 0) {
        continue;
      }

      int colonpos = token.indexOf(ProtocolConstants.INBOUND_MAP_ENTRY);
      if (colonpos == -1) {
        throw new MarshallException(paramType, Messages.getString("BeanConverter.MissingSeparator", //$NON-NLS-1$
            ProtocolConstants.INBOUND_MAP_ENTRY,
            token));
      }

      String key = token.substring(0, colonpos).trim();
      String val = token.substring(colonpos + 1).trim();
      tokens.put(key, val);
    }

    return tokens;
  }

  private ConverterManager converterManager;

  @Override
  public void setConverterManager(ConverterManager converterManager) {
    this.converterManager = converterManager;
  }

  /**
  * Accessor for the current ConverterManager
  * @return the current ConverterManager
  */
  public ConverterManager getConverterManager() {
    return converterManager;
  }

  @Override
  public Object convertInbound(
      @SuppressWarnings({ "rawtypes" }) Class paramType,
      InboundVariable data,
      InboundContext inctx) throws MarshallException {

    String value = data.getValue();

    // If the text is null then the whole bean is null
    if (value.trim().equals(ProtocolConstants.INBOUND_NULL)) {
      return null;
    }

    if (!value.startsWith(ProtocolConstants.INBOUND_MAP_START)) {
      throw new MarshallException(paramType, Messages.getString("BeanConverter.FormatError", //$NON-NLS-1$
          ProtocolConstants.INBOUND_MAP_START));
    }

    if (!value.endsWith(ProtocolConstants.INBOUND_MAP_END)) {
      throw new MarshallException(paramType, Messages.getString("BeanConverter.FormatError", //$NON-NLS-1$
          ProtocolConstants.INBOUND_MAP_START));
    }

    value = value.substring(1, value.length() - 1);

    final Map<String, String> tokens = extractInboundTokens(paramType, value);

    return convertInbound(paramType, data, tokens, inctx, value);
  }

  protected abstract Object convertInbound(
      @SuppressWarnings("rawtypes") Class paramType,
      InboundVariable data,
      Map<String, String> tokens,
      InboundContext inctx,
      String value) throws MarshallException;

  @Override
  public OutboundVariable convertOutbound(Object data, OutboundContext outctx)
      throws MarshallException {
    // TODO Auto-generated method stub, remove suppress warning if needed after real implementation
    return null;
  }

  protected <T> T getValue(
      String name,
      Class<T> type,
      InboundVariable data,
      Map<String, String> tokens,
      InboundContext inctx) throws MarshallException {
    final T value;
    final String valueString = tokens.get(name);
    if (valueString != null) {
      final String[] split = ParseUtil.splitInbound(valueString);
      final String splitValue = split[LocalUtil.INBOUND_INDEX_VALUE];
      final String splitType = split[LocalUtil.INBOUND_INDEX_TYPE];

      final InboundVariable nested = new InboundVariable(
          data.getLookup(),
          null,
          splitType,
          splitValue);

      @SuppressWarnings("unchecked")
      final T valueDraft = (T) getConverterManager().convertInbound(type, nested, inctx, null);

      value = valueDraft;
    }
    else {
      value = null;
    }
    return value;
  }

}
