/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.tools.jts;

import java.text.MessageFormat;

import org.directwebremoting.extend.MarshallException;

import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;

public class MultiPointConverter extends AbstractGeometryConverter<MultiPoint, double[][]> {

  private final PointConverter converter = new PointConverter();

  @Override
  protected String getGeometryType() {
    return "MultiPoint";
  }

  @Override
  protected Class<double[][]> getCoordinatesType() {
    return double[][].class;
  }

  @Override
  protected double[][] createCoordinates(MultiPoint geometry) throws MarshallException {

    if (geometry == null) {
      return null;
    }
    else if (geometry.isEmpty()) {
      return new double[0][0];
    }
    else {
      final double[][] coordinates = new double[geometry.getNumGeometries()][];
      for (int index = 0; index < coordinates.length; index++) {
        coordinates[index] = converter.createCoordinates((Point) geometry.getGeometryN(index));
      }
      return coordinates;
    }
  }

  @Override
  protected MultiPoint createGeometry(double[][] coordinates) throws MarshallException {

    if (coordinates == null) {
      return null;
    }
    else if (coordinates.length > 0) {
      final Point[] geometries = new Point[coordinates.length];
      for (int index = 0; index < coordinates.length; index++) {
        geometries[index] = converter.createGeometry(coordinates[index]);
      }
      return getGeometryFactory().createMultiPoint(geometries);
    }
    else {
      throw new MarshallException(MultiPoint.class, MessageFormat.format(
          "Incorrect number of coordinate components [{0}].", //$NON-NLS-1$
          coordinates.length));
    }
  }

}
