/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.disy.legato.tools.jts;

import org.directwebremoting.extend.MarshallException;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.LineString;
import com.vividsolutions.jts.geom.LinearRing;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;

public class PolygonConverter extends AbstractGeometryConverter<Polygon, double[][][]> {

  private final PointConverter pointConverter = new PointConverter();

  @Override
  protected String getGeometryType() {
    return "Polygon";
  }

  @Override
  protected Class<double[][][]> getCoordinatesType() {
    return double[][][].class;
  }

  @Override
  public double[][][] createCoordinates(Polygon polygon) throws MarshallException {

    if (polygon == null) {
      return null;
    }
    else if (polygon.isEmpty()) {
      return new double[0][0][0];
    }
    else {
      int numberOfPoints = polygon.getExteriorRing().getNumPoints();
      double[][][] coordinates = new double[1 + polygon.getNumInteriorRing()][numberOfPoints][];
      for (int index = 0; index < numberOfPoints; index++) {
        coordinates[0][index] = pointConverter.createCoordinates(polygon
            .getExteriorRing()
            .getPointN(index));
      }
      for (int i = 0; i < polygon.getNumInteriorRing(); i++) {
        LineString interiorRingN = polygon.getInteriorRingN(i);
        double[][] holeCoordinates = new double[interiorRingN.getNumPoints()][];
        for (int j = 0; j < interiorRingN.getNumPoints(); j++) {
          holeCoordinates[j] = pointConverter.createCoordinates(interiorRingN.getPointN(j));

        }
        coordinates[i + 1] = holeCoordinates;
      }
      return coordinates;
    }
  }

  @Override
  public Polygon createGeometry(double[][][] coordinates) throws MarshallException {
    if (coordinates == null || coordinates[0] == null) {
      return null;
    }
    else {
      Coordinate[] coordinateArray = new Coordinate[coordinates[0].length];
      for (int index = 0; index < coordinates[0].length; index++) {
        coordinateArray[index] = pointConverter
            .createGeometry(coordinates[0][index])
            .getCoordinate();
      }
      LinearRing shell = new LinearRing(
          new CoordinateArraySequence(coordinateArray),
          getGeometryFactory());
      LinearRing[] holes = new LinearRing[coordinates.length - 1];
      for (int j = 1; j < coordinates.length; j++) {
        coordinateArray = new Coordinate[coordinates[j].length];
        for (int index = 0; index < coordinates[j].length; index++) {
          coordinateArray[index] = pointConverter
              .createGeometry(coordinates[j][index])
              .getCoordinate();
        }
        holes[j - 1] = new LinearRing(
            new CoordinateArraySequence(coordinateArray),
            getGeometryFactory());
      }
      return getGeometryFactory().createPolygon(shell, holes);
    }
  }

}
