/**
 * Legato is a configurable, lightweight web mapping client that can be
 * easily embedded into web pages and portals, CMS and individual web
 * applications. Legato is implemented in JavaScript and based on the
 * popular open source library OpenLayers.
 *
 * Copyright (C) 2010  disy Informationssysteme GmbH, http://www.disy.net
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Copyright (c) 2008 by disy Informationssysteme GmbH
package net.disy.legato.tools.servlet;

import java.io.IOException;
import java.io.InputStream;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;

// NOT_PUBLISHED
public class ResourceServlet extends HttpServlet {

	private static final long serialVersionUID = -8451830693210613895L;
	private static final String RESOURCE_PARAMETER_NAME = "RESOURCE"; //$NON-NLS-1$

	@Override
	protected void service(final HttpServletRequest request,
			final HttpServletResponse response) throws IOException {

		final String resource = request.getParameter(RESOURCE_PARAMETER_NAME);

		if (resource == null) {
			response.sendError(HttpServletResponse.SC_NOT_FOUND,
					"Resource parameter is not specified."); //$NON-NLS-1$
		} else {

			final InputStream is;

			if (resource.startsWith("/")) { //$NON-NLS-1$
				is = getClass().getClassLoader().getResourceAsStream(
						resource.substring(1));
			} else {
				is = getClass().getResourceAsStream(resource);
			}

			if (is == null) {
				response.sendError(HttpServletResponse.SC_NOT_FOUND,
						"Resource [" //$NON-NLS-1$
								+ resource + "] could not be found."); //$NON-NLS-1$
			} else {
				renderResource(request, response, is);
			}
		}
	}

	protected void renderResource(final HttpServletRequest request,
			final HttpServletResponse response, final InputStream is)
			throws IOException {
		try {
			IOUtils.copy(is, response.getOutputStream());
		} finally {
			IOUtils.closeQuietly(is);
		}
	}
}
